
#include <libsaria/audio.h>
#include <libsaria/controls.h>
#include <libsaria/print.h>
#include <libsaria/library.h>
#include <libsaria/callback.h>
#include "audio.h"

GstElement *player = NULL;
GstElement *alsa = NULL;
GstBus *bus = NULL;

static void parse_error(GstMessage *error)
{
	GError *err;
	gchar  *debug;

	gst_message_parse_error(error, &err, &debug);
	g_print("Error: %s\n", err->message);
	g_error_free(err);
	g_free(debug);
}

static void notify_audio_changed()
{
	println("Notify audio changed");
	if (libsaria::get_pause_after()) {
		libsaria::audio::pause();
		libsaria::set_pause_after(false);
	}
	trigger_callback(TRACK_LOADED);
}

static gboolean on_message(GstBus *bus, GstMessage *message, gpointer data)
{
	switch (GST_MESSAGE_TYPE(message)) {
		/*
		 * The application message is only sent
		 * when the audio changes
		 */
		case GST_MESSAGE_APPLICATION:
			notify_audio_changed();
			break;
		case GST_MESSAGE_ERROR:
			parse_error(message);
		case GST_MESSAGE_EOS:
			libsaria::next();
		default:
			break;
	}
	return TRUE;
}

static void about_to_finish(GstElement *playbin, gpointer data)
{
	string file;

	println("About to finish!");
	file = libsaria::next_file();
	if (file != "")
		load_file(playbin, file);
}

static void audio_changed(GstElement *playbin, gpointer data)
{
	GQuark quark = g_quark_from_string("audio-changed");
	GstStructure *msg_struct = gst_structure_id_empty_new(quark);
	GstMessage *message = gst_message_new_application(GST_OBJECT(playbin), msg_struct);

	println("Audio changed!");
	gst_bus_post(bus, message);
}

namespace libsaria
{

	void audio::init(int argc, char **argv)
	{
		println("Initializing audio driver");
		gst_init(&argc, &argv);

		player = gst_element_factory_make("playbin2", "player");
		alsa   = gst_element_factory_make("alsasink", "alsa");
		bus    = gst_pipeline_get_bus(GST_PIPELINE(player));

		gst_bus_add_watch(bus, on_message, NULL);

		/* Without this gapless playback won't work */
		g_object_set(G_OBJECT(player), "audio-sink", alsa, NULL);

		g_signal_connect(player, "about-to-finish", G_CALLBACK(about_to_finish), NULL);
		g_signal_connect(player, "audio-changed", G_CALLBACK(audio_changed), NULL);

		init_volume();
	}

};
