
#include <libsaria/path.h>
#include <libsaria/index.h>
#include <libsaria/print.h>
#include "../list/list.h"
#include "path.h"

#include <dirent.h>
#include <list>
using namespace std;

#define MAX_SCAN 25

class ScanTask : public IdleTask
{
	private:
		LibraryPath *lib_path;
		list<pair<string, sid_t> > files;

	public:
		ScanTask(LibraryPath *path)
		{
			lib_path = path;
		}
		~ScanTask() {};

		unsigned int add_file(string &file, sid_t &sid)
		{
			files.push_back(pair<string, sid_t>(file, sid));
			return files.size();
		};

		void run_task();
};

class ReaddirTask : public IdleTask
{
	private:
		LibraryPath *lib_path;
		ScanTask *read_dir(string, ScanTask *);

	public:
		ReaddirTask(LibraryPath *path)
		{
			lib_path = path;
		};
		~ReaddirTask() {};
		void run_task();
};

static ScanTask *scandir(string dir, LibraryPath *lib_path, ScanTask *scan);

void ScanTask::run_task()
{
	list<pair<string, sid_t> >::iterator it;
	SaveTask *save;

	for (it = files.begin(); it != files.end(); it++) {
		try {
			lib_path->add_track(it->first, it->second);
		} catch (string message) {
			println(message);
		}
	}

	/* Save the library */
	save = new SaveTask(libsaria::library::save);
	save->queue_front();

	/* Bookkeeping for the UI */
	libsaria::library::sort_list();
	libsaria::index::refresh();
	libsaria::library::refresh();
}

static inline ScanTask *scan_entry(struct dirent *dirp,
				   LibraryPath *lib_path,
				   string &dir,
				   ScanTask *scan)
{
	string name;

	if (dirp == NULL)
		return scan;
	name = dirp->d_name;

	if (name == "." || name == "..")
		return scan;
	name = dir + "/" + name;

	switch(dirp->d_type) {
	case DT_DIR:
		scan = scandir(name, lib_path, scan);
		break;
	default:
		if (scan->add_file(name, dirp->d_ino) > MAX_SCAN) {
			scan->queue();
			scan = new ScanTask(lib_path);
		}
	}
	return scan;
}

static ScanTask *scandir(string dir, LibraryPath *lib_path, ScanTask *scan)
{
	DIR *dp;
	struct dirent *dirp;

	dp = opendir(dir.c_str());
	if (dp == NULL)
		return scan;

	do {
		dirp = readdir(dp);
		scan = scan_entry(dirp, lib_path, dir, scan);
	} while (dirp != NULL);

	closedir (dp);
	return scan;
}

void ReaddirTask::run_task()
{
	class ScanTask *scan = new ScanTask(lib_path);
	scan = scandir(lib_path->get_path(), lib_path, scan);
	scan->queue();
}

void LibraryPath::add_track(string &filepath, sid_t &songid)
{
	libsaria::Track *track;
	if (has_id(songid) == false) {
		track = new libsaria::Track(filepath, songid);
		file_list.push_back(track);
	}
}

void LibraryPath::update()
{
	ReaddirTask *task = new ReaddirTask(this);
	task->queue();
}
