
#include <libsaria/track.h>
#include <libsaria/format.h>
#include <libsaria/library.h>
#include <libsaria/index.h>

#include <taglib/tag.h>
#include <taglib/fileref.h>
#include <sstream>

namespace libsaria
{

	Track::Track()
	{
	}

	Track::Track(string file, sid_t sid)
	{
		TagLib::Tag *tag;
		TagLib::AudioProperties *prop;
		TagLib::FileRef ref(file.c_str());

		if (ref.isNull())
			throw "Error tagging file: " + file;

		filepath = file;
		songid   = sid;

		/* Extract tags */
		tag     = ref.tag();
		title   = tag->title().to8Bit(true);
		artist  = tag->artist().to8Bit(true);
		album   = tag->album().to8Bit(true);
		comment = tag->comment().to8Bit(true);
		genre   = tag->genre().to8Bit(true);
		year    = tag->year();
		track   = tag->track();

		/* Extract audio properties */
		prop     = ref.audioProperties();
		length   = prop->length();
		bitrate  = prop->bitrate();
		sample   = prop->sampleRate();
		channels = prop->channels();

		make_lenstr();
		do_bookkeeping();
	}

	Track::Track(InFile &in)
	{
		in >> songid;
		in >> filepath;
		in >> title;
		in >> artist;
		in >> album;
		in >> comment;
		in >> genre;
		in >> lenstr;
		in >> year;
		in >> track;
		in >> length;
		in >> bitrate;
		in >> sample;
		in >> channels;
		do_bookkeeping();
	}

	Track::~Track() {}

	void Track::do_cleanup()
	{
		libsaria::library::unlist_track(this);
	}

	void Track::do_bookkeeping()
	{
		artist_lc = lowercase(artist);
		album_lc = lowercase(album);
		libsaria::index::add_track(this);
		libsaria::library::list_track(this);
	}

	void Track::save(OutFile &out)
	{
		out << songid;
		out << filepath;
		out << title;
		out << artist;
		out << album;
		out << comment;
		out << genre;
		out << lenstr;
		out << year;
		out << track;
		out << length;
		out << bitrate;
		out << sample;
		out << channels;
		out << "\n";
	}

	void Track::make_lenstr()
	{
		int minutes;
		int seconds;
		stringstream stream;

		/* Convert length into mm:ss format */
		minutes = length / 60;
		seconds = length - (minutes * 60);
		stream << minutes << ":";
		if (seconds < 10)
			stream << "0";
		stream << seconds;
		lenstr = stream.str();
	}

} /* Namespace: libsaria */
